/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  led.c

  This modules supports a 2 digit 7 segment common anode display.
-----------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include "common.h"
#include "led.h"

extern volatile uint8_t	 ucCounter;		// activation count
extern volatile char cDisplay[2];		// digits to display

/*---------------------------------------------------------------------
  ledSelectDigit(...)

  Selects a digit of the led display. There are 2 digits selected by
  ports that drive the common pin of each display.

  Input: a single numeric digit, range 0..1. 0 drives the high order
         digit, 1 drives the low order digit. To turn off all digits,
         pass NO_DIGIT.
---------------------------------------------------------------------*/
void ledSelectDigit( char cDigit )
{

  switch ( cDigit )
  {
    case 0:
      clear_bit( LED_DIGIT_PORT, LED_DIGIT_1 );
      break;
    case 1:
      clear_bit( LED_DIGIT_PORT, LED_DIGIT_2 );
      break;
    case NO_DIGIT:
      set_bit( LED_DIGIT_PORT, LED_DIGIT_1 );
      set_bit( LED_DIGIT_PORT, LED_DIGIT_2 );
  }
}

/*---------------------------------------------------------------------
  ledOutputNumber(...)

  Outputs a number to the led display. Ports PA0..PA3 are connected
  to the A-B-C-D inputs of a 74LS47 which decodes the number and
  activates the corresponding segments of the bank of LEDs. 

  Input: a single numeric digit, range 0..9 (ascii or binary, only the
         low-order nibble is processed.)
---------------------------------------------------------------------*/
void ledOutputNumber( char cNumber )
{
  // Output the digit as BCD on PA0..PA3

  if ( test_bit( cNumber, 0 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_A );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_A );

  if ( test_bit( cNumber, 1 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_B );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_B );

  if ( test_bit( cNumber, 2 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_C );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_C );

  if ( test_bit( cNumber, 3 ) )
    set_bit( LED_ENCODER_PORT, LED_ENCODER_D );
  else
    clear_bit( LED_ENCODER_PORT, LED_ENCODER_D );
}

/*---------------------------------------------------------------------
  format_counter()

  Format our counter for the 2 digit led display.
---------------------------------------------------------------------*/
void format_counter()
{
  char  buffer[6];
  uint8_t  nn;

  if ( ucCounter > 99 )
  {
    cDisplay[0] = '9';
    cDisplay[1] = '9';
    return;
  }

  utoa( (uint16_t) ucCounter, buffer, 10 );
  nn = (uint8_t) strlen( buffer );
  if ( !nn )
  {
    cDisplay[0] = 0x0f;
    cDisplay[1] = '0';
  }
  else
    if ( nn == 1 )
    {
      cDisplay[0] = 0x0f;
      cDisplay[1] = buffer[0];
    }
    else
    {
      cDisplay[0] = buffer[0];
      cDisplay[1] = buffer[1];
    }
}
